@@ -65,5 +65,6 @@ gmt begin ex31
 	# plot legend
 	gmt legend -DjTR+o0.1c+w8.0c+l1.2 -C0.3c/0.4c -F+p+gwhite legend.txt
 
-	rm -f PSL_custom_fonts.txt legend.txt ex31CropNoLogo.eps europe-capitals-ru.csv
+	rm -f legend.txt ex31CropNoLogo.eps europe-capitals-ru.csv
 gmt end show
+rm -f PSL_custom_fonts.txt
