@@ -1,5 +1,6 @@
 #!/usr/bin/env bash
 gmt begin GMT_App_M_2
+gmt set GMT_THEME cookbook
 gmt set FONT_ANNOT_PRIMARY 10p PROJ_LENGTH_UNIT cm PS_MEDIA 11ix8.5i
 
 # Set up color palette with named annotations
@@ -17,7 +18,7 @@ cat > ages.cpt <<END
 359	104	255	0	416	104	255	0	;Devonian
 416	220	255	0	444	220	255	0	;Silurian
 444	255	174	0	488	255	174	0	;Ordovician
-488	255	58	0	542	255	58	0	;Cambrian
+488	255	58	0	542	255	58	0	;Cambrian;Precambrian
 B	black
 F	white
 END
