@@ -2,8 +2,8 @@
 #
 # Plot psxy text symbol for the man page
 
-ps=GMT_base_symbols3.ps
-
+gmt begin GMT_base_symbols3
+    gmt set GMT_THEME cookbook
 cat << EOF > tmp.txt
 # All the basic geometric psxy symbols
 > -Gblack
@@ -17,5 +17,6 @@ cat << EOF > tmp.txt
 > -Gred -Wfaint
 5	1	2c l+tp+f1c,Symbol
 EOF
-grep -v '>' tmp.txt | gmt psxy -R0.5/5.5/0.5/1.5 -B0g1 -B+n -Jx2c -Sc1.5c -W0.25p -P -K --PROJ_LENGTH_UNIT=cm -i0,1 --MAP_GRID_PEN_PRIMARY=default,dashed > $ps
-gmt psxy tmp.txt -R -J -S -Glightred -W1p -O --PROJ_LENGTH_UNIT=cm >> $ps
+grep -v '>' tmp.txt | gmt plot -R0.5/5.5/0.5/1.5 -B0g1 -B+n -Jx2c -Sc1.5c -W0.25p --PROJ_LENGTH_UNIT=cm -i0,1 --MAP_GRID_PEN_PRIMARY=default,dashed
+gmt plot tmp.txt -S -Glightred -W1p --PROJ_LENGTH_UNIT=cm
+gmt end show
