@@ -24,7 +24,7 @@ cat << EOF > tt.awk
 }
 EOF
 # Need access to PS source files for this plot
-GMT_SHAREDIR=`gmt --show-sharedir`
+GMT_SHAREDIR=$(gmt --show-sharedir)
 egrep -v '\[|\]' "${GMT_SHAREDIR}"/share/postscriptlight/$1.ps | $AWK -f tt.awk > tt.chart
 cat << EOF > tt.awk
 # This awk script creates a file for gmt plot to plot a rectangle for undefined entries
@@ -55,6 +55,7 @@ BEGIN {
 EOF
 
 gmt begin GMT_encoding
+gmt set GMT_THEME cookbook
 gmt set PS_CHAR_ENCODING $1
 gmt plot -R0/9/-1/32 -Jx0.345i/-0.21i -Bg1 -B+t"Octal codes for $1" -Ggray -X3i -Sri tt.empty
 $AWK -f tt.awk tt.chart | gmt text -F+f10p,Times-Roman
