@@ -2,7 +2,7 @@
 #
 # Show a basic seismicity legend
 #
-gmt begin GMT_seislegend ps
+gmt begin GMT_seislegend
 	gmt set GMT_THEME cookbook
 	gmt set FONT_ANNOT_PRIMARY 10p FONT_TITLE 18p FORMAT_GEO_MAP ddd:mm:ssF
 
