@@ -1,4 +1,4 @@
-"/ISO-8859-8_Encoding [\n\
+"/ISO-8859-15_Encoding [\n\
 /.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef\n\
 /.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef\n\
 /.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef\n\
@@ -19,16 +19,16 @@
 /.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef\n\
 /.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef\n\
 /.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef\n\
-/space		/.notdef	/cent		/sterling	/currency	/yen		/brokenbar	/section\n\
-/dieresis	/copyright	/multiply	/guillemotleft	/logicalnot	/hyphen		/registered	/.notdef\n\
-/degree		/plusminus	/twosuperior	/threesuperior	/acute		/mu		/paragraph	/periodcentered\n\
-/cedilla	/onesuperior	/divide		/guillemotright	/onequarter	/onehalf	/threequarters	/.notdef\n\
-/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef\n\
-/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef\n\
-/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef\n\
-/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef	/underscoredbl\n\
-/afii57664	/afii57665	/afii57666	/afii57667	/afii57668	/afii57669	/afii57670	/afii57671\n\
-/afii57672	/afii57673	/afii57674	/afii57675	/afii57676	/afii57677	/afii57678	/afii57679\n\
-/afii57680	/afii57681	/afii57682	/afii57683	/afii57684	/afii57685	/afii57686	/afii57687\n\
-/afii57688	/afii57689	/afii57690	/.notdef	/.notdef	/.notdef	/.notdef	/.notdef\n\
+/uni00A0	/exclamdown	/cent		/sterling	/Euro		/yen		/Scaron		/section\n\
+/scaron		/copyright	/ordfeminine	/guillemotleft	/logicalnot	/uni00AD	/registered	/macron\n\
+/degree		/plusminus	/twosuperior	/threesuperior	/Zcaron		/mu		/paragraph	/periodcentered\n\
+/zcaron		/onesuperior	/ordmasculine	/guillemotright	/OE		/oe		/Ydieresis	/questiondown\n\
+/Agrave		/Aacute		/Acircumflex	/Atilde		/Adieresis	/Aring		/AE		/Ccedilla\n\
+/Egrave		/Eacute		/Ecircumflex	/Edieresis	/Igrave		/Iacute		/Icircumflex	/Idieresis\n\
+/Eth		/Ntilde		/Ograve		/Oacute		/Ocircumflex	/Otilde		/Odieresis	/multiply\n\
+/Oslash		/Ugrave		/Uacute		/Ucircumflex	/Udieresis	/Yacute		/Thorn		/germandbls\n\
+/agrave		/aacute		/acircumflex	/atilde		/adieresis	/aring		/ae		/ccedilla\n\
+/egrave		/eacute		/ecircumflex	/edieresis	/igrave		/iacute		/icircumflex	/idieresis\n\
+/eth		/ntilde		/ograve		/oacute		/ocircumflex	/otilde		/odieresis	/divide\n\
+/oslash		/ugrave		/uacute		/ucircumflex	/udieresis	/yacute		/thorn		/ydieresis\n\
 ] def\n",
