@@ -1,6 +1,6 @@
 /*--------------------------------------------------------------------
  *
- *	Copyright (c) 1991-2021 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
+ *	Copyright (c) 1991-2024 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -37,6 +37,14 @@ enum GMT_enum_contline {
 	GMT_CONTOUR_XLINE,	/* Contour labels where crossing straight lines (via key points) */
 	GMT_CONTOUR_XCURVE};	/* Contour labels where crossing arbitrary lines (via file) */
 
+/*! Various settings for angles related to contours and decorated lines */
+enum GMT_enum_contangle {
+	GMT_ANGLE_LINE_PARALLEL = 0,	/* Angles follows the line locally */
+	GMT_ANGLE_LINE_NORMAL,	/* Angles is normal to the line locally */
+	GMT_ANGLE_LINE_FIXED,	/* Angle is fixed regardless of line direction */
+	GMT_ANGLE_LINE_DELTA};	/* Angle is a fixed offset from line direction */
+
+
 /*! Various settings for quoted line/contour label types */
 enum GMT_enum_label {
 	GMT_LABEL_IS_NONE = 0,	/* No contour/line crossing  */
