@@ -21,6 +21,7 @@
 "gmt/rainbow        : Rainbow, magenta-blue-cyan-green-yellow-red [C=HSV]",
 "gmt/red2green      : Polar scale from red to green via white [S, C=RGB]",
 "gmt/relief         : Wessel/Martinez colors for topography [R=-8000/+8000, H, C=RGB]",
+"gmt/rust2silver    : USGS color map for planets such as Mercury [R=-5500/5500, C=RGB]",
 "gmt/seafloor       : Purple-blue-white bathymetry scale [R=-6000/0, C=RGB]",
 "gmt/sealand        : Smith bathymetry/topography scale [R=-6000/+3000, H, C=HSV]",
 "gmt/seis           : R-O-Y-G-B seismic tomography colors [C=RGB]",
@@ -55,6 +56,7 @@
 "SCM/devon          : Perceptually uniform sequential colormap, by Fabio Crameri [C=RGB]",
 "SCM/devonS         : Perceptually uniform sequential categorical colormap, by Fabio Crameri [C=RGB]",
 "SCM/fes            : Perceptually uniform multi-sequential colormap by Fabio Crameri [H,C=RGB]",
+"SCM/glasgow        : Perceptually uniform sequential colormap by Fabio Crameri [C=RGB]",
 "SCM/grayC          : Perceptually uniform \'gray\' colormap, by Fabio Crameri [C=RGB]",
 "SCM/grayCS         : Perceptually uniform \'gray\' categorical colormap by Fabio Crameri [C=RGB]",
 "SCM/hawaii         : Perceptually uniform sequential colormap, by Fabio Crameri [C=RGB]",
@@ -65,7 +67,10 @@
 "SCM/lajollaS       : Perceptually uniform categorical colormap, without black or white, by Fabio Crameri [C=RGB]",
 "SCM/lapaz          : Perceptually uniform \'rainbow\' colormap, by Fabio Crameri [C=RGB]",
 "SCM/lapazS         : Perceptually uniform \'rainbow\' categorical colormap by Fabio Crameri [C=RGB]",
+"SCM/lipari         : Perceptually uniform sequential colormap by Fabio Crameri [C=RGB]",
 "SCM/lisbon         : Perceptually uniform bimodal colormap, dark, by Fabio Crameri [S, C=RGB]",
+"SCM/managua        : Perceptually uniform diverging colormap, by Fabio Crameri [S,C=RGB]",
+"SCM/navia          : Perceptually uniform sequential colormap by Fabio Crameri [C=RGB]",
 "SCM/nuuk           : Perceptually uniform, low-lightness gradient colormap, by Fabio Crameri [C=RGB]",
 "SCM/nuukS          : Perceptually uniform, low-lightness gradient categorical colormap, by Fabio Crameri [C=RGB]",
 "SCM/oleron         : Perceptually uniform topography colormap, by Fabio Crameri [H, C=RGB]",
