@@ -1,6 +1,6 @@
 /*--------------------------------------------------------------------
  *
- *	Copyright (c) 1991-2022 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
+ *	Copyright (c) 1991-2024 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -1177,7 +1177,7 @@ void gmtlib_encode (struct GMT_CTRL *GMT, void *vptr, uint64_t k, gmt_grdfloat z
 			((int *)vptr)[k] = (int)lrintf (z);
 			break;
 		case 'f':
-			((float *)vptr)[k] = (float)z;
+			((gmt_grdfloat *)vptr)[k] = z;
 			break;
 		case 'd':
 			((double *)vptr)[k] = (double)z;
