@@ -1,6 +1,6 @@
 /*--------------------------------------------------------------------
  *
- *	Copyright (c) 1991-2022 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
+ *	Copyright (c) 1991-2024 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -20,13 +20,15 @@
 
 /* Misc functions to find and read DCW polygons.
  * Some of the countries have state borders too.
- * The PUBLIC functions are (5):
+ *
+ * A) List of exported gmt_* functions available to modules and libraries via gmt_dev.h:
  *
  * gmt_DCW_option	: Present the DCW option and modifier usage
  * gmt_DCW_parse	: Parse the DCW option arguments
  * gmt_DCW_list 	: List the available polygons and exit
  * gmt_DCW_operation	: Get DCW polygons and operate on them
  * gmt_DCW_free		: Free memory allocated by gmt_DCW_parse
+ * gmt_DCW_version	: Return DCW version
  *
  * Author:	Paul Wessel
  * Date:	1-MAY-2013
@@ -59,7 +61,7 @@ struct GMT_DCW_COUNTRY {	/* Information per country */
 
 struct GMT_DCW_STATE {		/* Information per state */
 	char country[4];	/* 2-char country code ISO 3166-1 (e.g., BR, US) */
-	char code[4];		/* 2/3-char state codes for US, Canada, China, Argentina, Australia, Brazil, Russia (e.g., TX) */
+	char code[4];		/* 2/3-char state codes for Argentina, Australia, Brazil, Canada, China, India, Norway, Russia and the US (e.g., TX) */
 	char name[80];		/* Full name of the state */
 };
 
@@ -180,6 +182,7 @@ GMT_LOCAL int gmtdcw_load_lists (struct GMT_CTRL *GMT, struct GMT_DCW_COUNTRY **
 	while ( gmt_fgets (GMT, line, BUFSIZ, fp)) {
 		if (line[0] == '#') continue;	/* Skip comments */
 		sscanf (line, "%s %s %[^\n]", Country[k].continent, Country[k].code,  Country[k].name);
+		gmt_chop (Country[k].name);	/* Eliminate any nasty CR/LF from Windows */
 		k++;
 		if (k == n_alloc) {
 			n_alloc += GMT_LEN128;
@@ -205,6 +208,7 @@ GMT_LOCAL int gmtdcw_load_lists (struct GMT_CTRL *GMT, struct GMT_DCW_COUNTRY **
 	while ( gmt_fgets (GMT, line, BUFSIZ, fp)) {
 		if (line[0] == '#') continue;	/* Skip comments */
 		sscanf (line, "%s %s %[^\n]", State[k].country, State[k].code,  State[k].name);
+		gmt_chop (State[k].name);	/* Eliminate any nasty CR/LF from Windows */
 		if (k && strcmp (State[k].country, State[k-1].country)) ns++;	/* New country with states */
 		k++;
 		if (k == n_alloc) {
@@ -260,6 +264,7 @@ GMT_LOCAL int gmtdcw_load_lists (struct GMT_CTRL *GMT, struct GMT_DCW_COUNTRY **
 				goto bail;
 			}
 			nf = sscanf (&line[5], "%s %[^\n]", Collection[k].region, Collection[k].name);
+			gmt_chop (Collection[k].name);	/* Eliminate any nasty CR/LF from Windows */
 			if (nf == 2 && !isalpha (Collection[k].name[0])) {	/* Gave a name that do not start with a letter */
 				GMT_Report (GMT->parent, GMT_MSG_ERROR, "Collection name must start with a letter (%s)\n", (Collection[k].name));
 				goto bail;
@@ -426,7 +431,7 @@ int gmt_DCW_version (struct GMTAPI_CTRL *API, char *version) {
 	int cdfid, err;
 	char path[PATH_MAX] = {""};
 	struct GMT_CTRL *GMT = API->GMT;
-	
+
 	if (version == NULL)
 		return (GMT_PTR_IS_NULL);
 
@@ -757,6 +762,18 @@ struct GMT_DATASET * gmt_DCW_operation (struct GMT_CTRL *GMT, struct GMT_DCW_SEL
 		if ((retval = nc_get_att_double (ncid, yvarid, "min", &south))) continue;
 		if ((retval = nc_get_att_double (ncid, yvarid, "max", &north))) continue;
 		if ((retval = nc_get_att_double (ncid, yvarid, "scale", &yscl))) continue;
+		if (GMT->current.setting.format_geo_out[0] == 'D') {	/* [-180 180]*/
+			if (west > 180) west -= 360;
+			if (east > 180) east -= 360;
+		}
+		else if (GMT->current.setting.format_geo_out[0] == '+') {	/* [0 360]*/
+			if (west < 0) west += 360;
+			if (east < 0) east += 360;
+		}
+		else if (GMT->current.setting.format_geo_out[0] == '-') {	/* [-360 0]*/
+			if (west > 0) west -= 360;
+			if (east > 0) east -= 360;
+		}
 		if (mode & GMT_DCW_REGION) {	/* Just update wesn */
 			Z[r_item].west = west;	Z[r_item++].east = east;
 			if (south < wesn[YLO]) wesn[YLO] = south;
@@ -793,7 +810,7 @@ struct GMT_DATASET * gmt_DCW_operation (struct GMT_CTRL *GMT, struct GMT_DCW_SEL
 		/* Extract the pieces into separate segments */
 		k = seg = 0;
 		done = false;
-	        while (!done) {
+		while (!done) {
 			first = GMT_NOTSET;
 			while (first == GMT_NOTSET && k < np) {	/* Look for next start of segment marker */
 				if (gmt_M_is_dnan (lon[k])) {
@@ -899,7 +916,8 @@ struct GMT_DATASET * gmt_DCW_operation (struct GMT_CTRL *GMT, struct GMT_DCW_SEL
 
 	if (special) {	/* Plot via psxy or clip via psclip, then free dataset */
 		char cmd[GMT_BUFSIZ] = {""}, in_string[GMT_VF_LEN] = {""};
-		static char *module[2] = {"psxy", "psclip"};
+		static char *module[4] = {"psxy", "psclip", "plot", "clip"};
+		if (GMT->current.setting.run_mode == GMT_MODERN) special += 2;	/* Use modern mode module names */
 		/* Get a virtual file for the current DCW dataset */
 		if (GMT_Open_VirtualFile (GMT->parent, GMT_IS_DATASET, GMT_IS_POLY, GMT_IN|GMT_IS_REFERENCE, D, in_string) == GMT_NOTSET) {
 			return (NULL);
@@ -1048,7 +1066,7 @@ void gmt_DCW_option (struct GMTAPI_CTRL *API, char option, unsigned int plot) {
 		GMT_Usage (API, 3, "+g Fill polygons using given <fill> to fill [none].");
 	}
 	GMT_Usage (API, 3, "+l Just list the countries and their codes [no %s takes place].", action2[plot]);
-	GMT_Usage (API, 3, "+L List states/territories for Argentina, Australia, Brazil, Canada, China, India, Russia and the US. "
+	GMT_Usage (API, 3, "+L List states/territories for Argentina, Australia, Brazil, Canada, China, India, Norway, Russia and the US. "
 		"Select =<continent>+l|L to only list countries from that continent or <code>+L for that country(repeatable).");
 	GMT_Usage (API, 3, "+n List collections and named regions, their codes and list of items (or region) [no %s takes place].", action2[plot]);
 	if (plot == 1)
