@@ -1,6 +1,6 @@
 /*--------------------------------------------------------------------
  *
- *	Copyright (c) 1991-2022 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
+ *	Copyright (c) 1991-2024 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -21,17 +21,19 @@
  * Author:	Joaquim Luis
  * Date:	27-Aug-2009
  * Revision: 1		Based on gdalread.c MEX from Mirone
- *			The populate_metadata() was modified from mexgdal of John Evans (johnevans@acm.org)
+ *			The gmtgdalread_populate_metadata() was modified from mexgdal of John Evans (johnevans@acm.org)
  *
- * Public functions (2):
+ * A) List of exported gmt_* functions available to modules and libraries via gmt_dev.h:
+ *	gmt_gdalread     : Read a GDAL grid
+ *
+ * B) List of exported gmtlib_* functions available to libraries via gmt_internals.h:
  *
  *	gmtlib_is_gdal_grid : Determine if a grid is GDAL readable
- *	gmt_gdalread     : Read a GDAL grid
  */
 
 /* Local functions */
 
-GMT_LOCAL GDALDatasetH gdal_open (struct GMT_CTRL *GMT, char *gdal_filename) {
+GMT_LOCAL GDALDatasetH gmtgdalread_gdal_open (struct GMT_CTRL *GMT, char *gdal_filename) {
 	char *file = NULL, path[PATH_MAX] = {""}, *c = NULL;
 	if (gmtlib_found_url_for_gdal (gdal_filename))	/* A vis*** URL, pass to GDAL as is */
 		strncpy (path, gdal_filename, PATH_MAX-1);
@@ -51,7 +53,7 @@ GMT_LOCAL GDALDatasetH gdal_open (struct GMT_CTRL *GMT, char *gdal_filename) {
 	return (GDALOpen (path, GA_ReadOnly));
 }
 
-GMT_LOCAL int gdal_decode_columns (struct GMT_CTRL *GMT, char *txt, int *whichBands) {
+GMT_LOCAL int gmtgdalread_decode_columns (struct GMT_CTRL *GMT, char *txt, int *whichBands) {
 	unsigned int n = 0, i, start, stop, pos = 0;
 	char p[GMT_LEN256];
 	gmt_M_unused(GMT);
@@ -70,12 +72,12 @@ GMT_LOCAL int gdal_decode_columns (struct GMT_CTRL *GMT, char *txt, int *whichBa
 }
 
 /* ---------------------------------------------------------------------------------
- * record_geotransform:
+ * gmtgdalread_record_geotransform:
  *
  * If the gdal file is not internally georeferenced, try to get the world file.
  * Returns -1 in case no world file is found.
  */
-GMT_LOCAL int record_geotransform (char *gdal_filename, GDALDatasetH hDataset, double *adfGeoTransform) {
+GMT_LOCAL int gmtgdalread_record_geotransform (char *gdal_filename, GDALDatasetH hDataset, double *adfGeoTransform) {
 	int status = GMT_NOTSET;
 	char generic_buffer[5000];
 
@@ -97,10 +99,10 @@ GMT_LOCAL int record_geotransform (char *gdal_filename, GDALDatasetH hDataset, d
 }
 
 /************************************************************************/
-/*                        ReportCorner()                                */
+/*                        gmtgdalread_ReportCorner()                                */
 /************************************************************************/
 
-GMT_LOCAL int ReportCorner (struct GMT_CTRL *GMT, GDALDatasetH hDataset, OGRCoordinateTransformationH hTransform,
+GMT_LOCAL int gmtgdalread_ReportCorner (struct GMT_CTRL *GMT, GDALDatasetH hDataset, OGRCoordinateTransformationH hTransform,
                             double x, double y, double *xy_c, double *xy_geo) {
 	double	dfGeoX, dfGeoY;
 	double	adfGeoTransform[6];
@@ -139,10 +141,10 @@ GMT_LOCAL int ReportCorner (struct GMT_CTRL *GMT, GDALDatasetH hDataset, OGRCoor
 }
 
 /* -------------------------------------------------------------------- */
-GMT_LOCAL void ComputeRasterMinMax(struct GMT_CTRL *GMT, unsigned char *tmp, GDALRasterBandH hBand, double adfMinMax[2],
+GMT_LOCAL void gmtgdalread_ComputeRasterMinMax(struct GMT_CTRL *GMT, unsigned char *tmp, GDALRasterBandH hBand, double adfMinMax[2],
 			int nXSize, int nYSize, double z_min, double z_max) {
 	/* Compute Min/Max of a sub-region. I'm forced to do this because the
-	GDALComputeRasterMinMax works only on the entire dataset */
+	gmtgdalread_ComputeRasterMinMax works only on the entire dataset */
 	int	i, bGotNoDataValue;
 	int16_t	tmpI16;
 	uint16_t	tmpUI16;
@@ -223,7 +225,7 @@ GMT_LOCAL void ComputeRasterMinMax(struct GMT_CTRL *GMT, unsigned char *tmp, GDA
 	adfMinMax[1] = z_max;
 }
 
-int get_attrib_from_string(struct GMT_GDALREAD_OUT_CTRL *Ctrl, GDALRasterBandH hBand, int nBand, double  *dfNoDataValue) {
+GMT_LOCAL int gmtgdalread_get_attrib_from_string(struct GMT_GDALREAD_OUT_CTRL *Ctrl, GDALRasterBandH hBand, int nBand, double  *dfNoDataValue) {
 	/* Since several methods to get band's attributes for HDF5 Datasets are not yet implemented in GDAL2.1.0
 	   namely GDALGetRasterScale() and friends, the temporary work-around is to fish them from the Metadata
 	   strings that we can access via GDALGetMetadata().
@@ -255,7 +257,7 @@ int get_attrib_from_string(struct GMT_GDALREAD_OUT_CTRL *Ctrl, GDALRasterBandH h
 	return (GMT_NOERROR);
 }
 
-GMT_LOCAL int populate_metadata (struct GMT_CTRL *GMT, struct GMT_GDALREAD_OUT_CTRL *Ctrl, char *gdal_filename, int got_R,
+GMT_LOCAL int gmtgdalread_populate_metadata (struct GMT_CTRL *GMT, struct GMT_GDALREAD_OUT_CTRL *Ctrl, char *gdal_filename, int got_R,
                                  int nXSize, int nYSize, double dfULX, double dfULY, double dfLRX, double dfLRY,
 								 double z_min, double z_max, int first_layer) {
 /* =============================================================================================== */
@@ -331,7 +333,7 @@ GMT_LOCAL int populate_metadata (struct GMT_CTRL *GMT, struct GMT_GDALREAD_OUT_C
 	/* ------------------------------------------------------------------------- */
 	/* Open the file (if we can). */
 	/* ------------------------------------------------------------------------- */
-	hDataset = gdal_open (GMT, gdal_filename);
+	hDataset = gmtgdalread_gdal_open (GMT, gdal_filename);
 	if (hDataset == NULL) {
 		GMT_Report (GMT->parent, GMT_MSG_ERROR, "Unable to open %s.\n", gdal_filename);
 		gmtlib_GDALDestroyDriverManager(GMT->parent);
@@ -375,7 +377,7 @@ GMT_LOCAL int populate_metadata (struct GMT_CTRL *GMT, struct GMT_GDALREAD_OUT_C
 	/* ------------------------------------------------------------------------- */
 	/* Record the geotransform. */
 	/* ------------------------------------------------------------------------- */
-	status = record_geotransform (gdal_filename, hDataset, adfGeoTransform);
+	status = gmtgdalread_record_geotransform (gdal_filename, hDataset, adfGeoTransform);
 	if (!strcmp(GDALGetDriverShortName(GDALGetDatasetDriver(hDataset)),"netCDF") && GDAL_VERSION_NUM <= 1450) {
 		adfGeoTransform[3] *= -1;
 		adfGeoTransform[5] *= -1;
@@ -504,7 +506,7 @@ GMT_LOCAL int populate_metadata (struct GMT_CTRL *GMT, struct GMT_GDALREAD_OUT_C
 		if (bSuccess == 0 && strstr(Ctrl->DriverShortName, "HDF5") != NULL) {	/* several methods for HDF5 driver are not implemented */
 			GMT_Report(GMT->parent, GMT_MSG_INFORMATION, "An HDF5 file. Trying to get scale_offset from string metadata.\n");
 			dfNoDataValue = GMT->session.d_NaN;
-			get_attrib_from_string(Ctrl, hBand, nBand, &dfNoDataValue);			/* Go get them from the metadata in strings. */
+			gmtgdalread_get_attrib_from_string(Ctrl, hBand, nBand, &dfNoDataValue);			/* Go get them from the metadata in strings. */
 			if (!isnan(dfNoDataValue)) got_noDataValue = true;
 		}
 	}
@@ -521,7 +523,7 @@ GMT_LOCAL int populate_metadata (struct GMT_CTRL *GMT, struct GMT_GDALREAD_OUT_C
 	/* ------------------------------------------------------------------------- */
 	/* Get the first band NoData Value */
 	/* ------------------------------------------------------------------------- */
-	if (!got_noDataValue) {		/* May have been found if get_attrib_from_string() was acalled above */
+	if (!got_noDataValue) {		/* May have been found if gmtgdalread_get_attrib_from_string() was acalled above */
 		dfNoDataValue = GDALGetRasterNoDataValue(hBand, &status);
 		if (status)
 			Ctrl->nodata = dfNoDataValue;
@@ -602,28 +604,28 @@ GMT_LOCAL int populate_metadata (struct GMT_CTRL *GMT, struct GMT_GDALREAD_OUT_C
 	}
 
 	if (!got_R)					/* Lower Left */
-		ReportCorner (GMT, hDataset, hTransform, 0.0, GDALGetRasterYSize(hDataset), xy_c, xy_geo[0]);
+		gmtgdalread_ReportCorner (GMT, hDataset, hTransform, 0.0, GDALGetRasterYSize(hDataset), xy_c, xy_geo[0]);
 	else
 		xy_c[0] = dfULX, xy_c[1] = dfLRY;
 	Ctrl->Corners.LL[0] = Ctrl->hdr[0] = xy_c[0];	/* xmin, ymin */
 	Ctrl->Corners.LL[1] = Ctrl->hdr[2] = xy_c[1];
 
 	if (!got_R)					/* Upper Left */
-		ReportCorner (GMT, hDataset, hTransform, 0.0, 0.0, xy_c, xy_geo[1]);
+		gmtgdalread_ReportCorner (GMT, hDataset, hTransform, 0.0, 0.0, xy_c, xy_geo[1]);
 	else
 		xy_c[0] = dfULX, xy_c[1] = dfULY;
 	Ctrl->Corners.UL[0] = xy_c[0];
 	Ctrl->Corners.UL[1] = xy_c[1];
 
 	if (!got_R)					/* Upper Right */
-		ReportCorner (GMT, hDataset, hTransform, GDALGetRasterXSize(hDataset), 0.0, xy_c, xy_geo[2]);
+		gmtgdalread_ReportCorner (GMT, hDataset, hTransform, GDALGetRasterXSize(hDataset), 0.0, xy_c, xy_geo[2]);
 	else
 		xy_c[0] = dfLRX, xy_c[1] = dfULY;
 	Ctrl->Corners.UR[0] = Ctrl->hdr[1] = xy_c[0];	/* xmax, ymax */
 	Ctrl->Corners.UR[1] = Ctrl->hdr[3] = xy_c[1];
 
 	if (!got_R)					/* Lower Right */
-		ReportCorner (GMT, hDataset, hTransform, GDALGetRasterXSize(hDataset), GDALGetRasterYSize(hDataset), xy_c, xy_geo[3]);
+		gmtgdalread_ReportCorner (GMT, hDataset, hTransform, GDALGetRasterXSize(hDataset), GDALGetRasterYSize(hDataset), xy_c, xy_geo[3]);
 	else
 		xy_c[0] = dfLRX, xy_c[1] = dfLRY;
 	Ctrl->Corners.LR[0] = xy_c[0];
@@ -698,7 +700,7 @@ GMT_LOCAL int populate_metadata (struct GMT_CTRL *GMT, struct GMT_GDALREAD_OUT_C
 
 	GDALClose (hDataset);
 
-	gmt_M_toc (GMT, "In populate_metadata");
+	gmt_M_toc (GMT, "In gmtgdalread_populate_metadata");
 
 	return (GMT_NOERROR);
 }
@@ -717,7 +719,7 @@ int gmtlib_is_gdal_grid (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header) {
 	if (strstr(HH->name, ".jp2") || strstr(HH->name, ".JP2"))
 		if ((hDriver = GDALGetDriverByName("JP2OpenJPEG")) != NULL && (hDriver = GDALGetDriverByName("JP2ECW")) != NULL)
 			GDALDeregisterDriver(hDriver);		/* Deregister the JP2ECW driver. That is, prefer the OpenJPEG one */
-	hDataset = gdal_open (GMT, HH->name);
+	hDataset = gmtgdalread_gdal_open (GMT, HH->name);
 
 	if (hDataset == NULL)
 		return (GMT_GRDIO_BAD_VAL);
@@ -759,8 +761,9 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 	bool   just_copy = false, copy_flipud = false, UDflip_Y = false;
 	int	   *whichBands = NULL;
 	int64_t *rowVec = NULL, *colVec = NULL;
-	int64_t  off, i_x_nXYSize, startColPos = 0, indent = 0, col_indent = 0, nXSize_withPad = 0, nYSize_withPad;
+	int64_t  off, i_x_nXYSize, startColPos = 0, indent = 0, nXSize_withPad = 0, nYSize_withPad;
 	int64_t  n_alloc, n, m, nn, mm, ij, layer_size, layer_offset = 0;
+	//int64_t  col_indent = 0;
 	unsigned char *tmp = NULL;
 	double  adfMinMax[2];
 	double	adfGeoTransform[6];
@@ -776,7 +779,7 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 
 	if (prhs->B.active) {		/* We have a selected bands request */
 		if ((whichBands = gmt_M_memory (GMT, NULL, 128, int)) == NULL) return GMT_NOTSET;		/* 128 is huge enough */
-		nReqBands = gdal_decode_columns (GMT, prhs->B.bands, whichBands);
+		nReqBands = gmtgdalread_decode_columns (GMT, prhs->B.bands, whichBands);
 	}
 	else if (prhs->f_ptr.active) {
 		/* Here we are going to read to a grid so if no band info was provided, default to read only the
@@ -903,9 +906,13 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 			GDALDeregisterDriver(hDriver);		/* Deregister the JP2ECW driver. That is, prefer the OpenJPEG one */
 
 	if (metadata_only) {	/* Just get the header info and return with it */
-		if (populate_metadata (GMT, Ctrl, gdal_filename, got_R, nXSize[0], nYSize, dfULX, dfULY, dfLRX, dfLRY, z_min, z_max, first_layer))
+		if (gmtgdalread_populate_metadata (GMT, Ctrl, gdal_filename, got_R, nXSize[0], nYSize, dfULX, dfULY, dfLRX, dfLRY, z_min, z_max, first_layer))
 			return(GMT_NOTSET);
 
+		/* See if uint8 images have set a noData */
+		if (Ctrl->band_field_names[0].nodata > -1e10)
+			Ctrl->nodata = Ctrl->band_field_names[0].nodata;
+
 		/* Return registration based on data type of first band. Byte is pixel reg otherwise set grid registration */
 		if (!Ctrl->hdr[6]) {		/* Grid registration */
 			Ctrl->hdr[0] += Ctrl->hdr[7] / 2;	Ctrl->hdr[1] -= Ctrl->hdr[7] / 2;
@@ -917,10 +924,10 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 
 	/* Read the image */
 
-	hDataset = gdal_open (GMT, gdal_filename);
+	hDataset = gmtgdalread_gdal_open (GMT, gdal_filename);
 
 	if (hDataset == NULL) {
-		GMT_Report (GMT->parent, GMT_MSG_ERROR, "gmt_gdalread: gdal_open failed %s\n", CPLGetLastErrorMsg());
+		GMT_Report (GMT->parent, GMT_MSG_ERROR, "gmt_gdalread: gmtgdalread_gdal_open failed %s\n", CPLGetLastErrorMsg());
 		gmt_M_free (GMT, whichBands);
 		return (GMT_NOTSET);
 	}
@@ -1055,6 +1062,9 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 
 	switch (GDALGetRasterDataType(hBand)) {
 		case GDT_Byte:
+			/* Because of a GDAL bug that misidentifies uint8 as GDT_CFloat64 (enum 14) we are forced to do this
+			   ugly patch. Make this conditional on a GDAL version when it is fixed upstream. */
+		case 14:
 			if (prhs->c_ptr.active)	/* We have a pointer with already allocated memory ready to use */
 				Ctrl->UInt8.data = prhs->c_ptr.grd;
 			else if ((Ctrl->UInt8.data = gmt_M_memory (GMT, NULL, n_alloc, uint8_t)) == NULL)
@@ -1214,7 +1224,7 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 
 				/* If we didn't compute it yet, its time to do it now (for this block) */
 				if (got_R) {	/* Update z_min, z_max in case we get here more than once (multiple blocks or two pieces) */
-					ComputeRasterMinMax(GMT, tmp, hBand, adfMinMax, nXSize[piece], buffy, z_min, z_max);
+					gmtgdalread_ComputeRasterMinMax(GMT, tmp, hBand, adfMinMax, nXSize[piece], buffy, z_min, z_max);
 					z_min = adfMinMax[0];	z_max = adfMinMax[1];
 				}
 
@@ -1242,6 +1252,9 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 
 				switch (GDALGetRasterDataType(hBand)) {
 					case GDT_Byte:
+						/* Because of a GDAL bug that misidentifies uint8 as GDT_CFloat64 we are forced to do this
+						   ugly patch. Make this conditional on a GDAL version when it is fixed upstream. */
+					case 14:
 						/* This chunk is kind of complicated because we want to take into account several different cases */
 						for (n = 0; n < nXSize[piece]; n++) {
 							if (do_BIP)			/* Vector for Pixel Interleaving */
@@ -1404,10 +1417,10 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 		gmt_M_free (GMT, rowVec);
 		if (!(just_copy || copy_flipud))
 			gmt_M_str_free (tmp);
-		col_indent = nXSize[piece];	/* The second time (if there is one) we must step in to pick up western section */
 		indent = pad_w[piece] + nXSize[piece];	/* The second time (if there is one) we must step in to pick up western section */
 		layer_offset += layer_size;	/* As of now, this is never used after increment for float images */
 #if 0	/* This code is problematic and commented out for now. PW, 5/15/2016 */
+		col_indent = nXSize[piece];	/* The second time (if there is one) we must step in to pick up western section */
 		if (Ctrl->Float.active && !isnan(prhs->N.nan_value)) {
 			for (m = startRow, mm = 0; m < endRow ; m++, mm++) {
 				nn = layer_offset + (pad_w[piece]+m)*nXSize_withPad + startColPos;
@@ -1425,12 +1438,12 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 
 	gmt_M_toc (GMT, "gmt_gdalread: After gdalread data reading");
 
-	populate_metadata (GMT, Ctrl, gdal_filename, got_R, nXSize[0]+nXSize[1], nYSize, dfULX, dfULY, dfLRX, dfLRY, z_min, z_max, first_layer);
+	gmtgdalread_populate_metadata (GMT, Ctrl, gdal_filename, got_R, nXSize[0]+nXSize[1], nYSize, dfULX, dfULY, dfLRX, dfLRY, z_min, z_max, first_layer);
 
 	gmtlib_GDALDestroyDriverManager(GMT->parent);
 
 	/* Return registration based on data type of the actually read first band.
-	   We do this at the end because 'populate_metadata' scans all bands in file
+	   We do this at the end because 'gmtgdalread_populate_metadata' scans all bands in file
 	   and cannot know which one was actually read. */
 	if (!pixel_reg) {		/* Grid registration */
 		Ctrl->hdr[0] += Ctrl->hdr[7] / 2;	Ctrl->hdr[1] -= Ctrl->hdr[7] / 2;
@@ -1441,4 +1454,3 @@ int gmt_gdalread (struct GMT_CTRL *GMT, char *gdal_filename, struct GMT_GDALREAD
 
 	return (GMT_NOERROR);
 }
-
