@@ -1,6 +1,6 @@
 /*--------------------------------------------------------------------
  *
- *	Copyright (c) 1991-2022 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
+ *	Copyright (c) 1991-2024 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -34,11 +34,16 @@
 #ifdef HAVE_GLIB_GTHREAD
 #include <glib.h>
 
+#define GMT_xg_OPT " [-x[[-]<n>]] "  /* Must add spaces and brackets here and place via %s since it may be blank */
+#define GMT_ADD_xg_OPT   "x"
+
 #define GMT_declare_gmutex static GMutex mutex;
 #define GMT_set_gmutex g_mutex_lock (&mutex);
 #define GMT_unset_gmutex g_mutex_unlock (&mutex);
 
-#else
+#else    /* No GLIB support */
+#define GMT_xg_OPT " "
+#define GMT_ADD_xg_OPT   ""
 
 #define GMT_declare_gmutex
 #define GMT_set_gmutex
