@@ -1,6 +1,6 @@
 /*--------------------------------------------------------------------
  *
- *	Copyright (c) 1991-2022 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
+ *	Copyright (c) 1991-2024 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -26,6 +26,18 @@
  * Version:	6.x
  */
 
+/*
+ * A) List of exported gmt_* functions available to modules and libraries via gmt_dev.h:
+ *
+ *	gmt_current_name
+ *	gmt_get_full_name
+ *
+ * B) List of exported gmtlib_* functions available to libraries via gmt_internals.h:
+ *	gmtlib_get_active_name
+ *	gmtlib_is_modern_name
+ *	gmtlib_set_KOP_strings
+ */
+
 #include "gmt_dev.h"
 
 const char *gmt_current_name (const char *module, char modname[]) {
@@ -58,12 +70,14 @@ const char *gmt_current_name (const char *module, char modname[]) {
 	else if (!strncmp (module, "rose",         4U)) { strcpy (modname, "psrose");      return module; }
 	else if (!strncmp (module, "mask",         4U)) { strcpy (modname, "psmask");      return module; }
 	else if (!strncmp (module, "clip",         4U)) { strcpy (modname, "psclip");      return module; }
+	else if (!strncmp (module, "barb",         4U)) { strcpy (modname, "psbarb");      return module; }
 	else if (!strncmp (module, "sac",          3U)) { strcpy (modname, "pssac");       return module; }
 	/* Then look for modules that now have a different modern mode name */
 	else if (!strncmp (module, "pshistogram", 11U)) { strcpy (modname, module); return "histogram"; }
 	else if (!strncmp (module, "psternary",    9U)) { strcpy (modname, module); return "ternary";   }
 	else if (!strncmp (module, "pscontour",    9U)) { strcpy (modname, module); return "contour";   }
 	else if (!strncmp (module, "psbasemap",    9U)) { strcpy (modname, module); return "basemap";   }
+	else if (!strncmp (module, "psbarb",       6U)) { strcpy (modname, module); return "barb";      }
 	else if (!strncmp (module, "psevents",     8U)) { strcpy (modname, module); return "events";    }
 	else if (!strncmp (module, "pswiggle",     8U)) { strcpy (modname, module); return "wiggle";    }
 	else if (!strncmp (module, "pslegend",     8U)) { strcpy (modname, module); return "legend";    }
@@ -107,6 +121,7 @@ const char *gmt_get_full_name (struct GMTAPI_CTRL *API, const char *module) {
 	else if (!strcmp (module, "set"))       return "gmtset";
 	else if (!strcmp (module, "simplify"))  return "gmtsimplify";
 	else if (!strcmp (module, "spatial"))   return "gmtspatial";
+	else if (!strcmp (module, "split"))     return "gmtsplit";
 	else if (!strcmp (module, "vector"))    return "gmtvector";
 	else if (!strcmp (module, "which"))     return "gmtwhich";
 	else if (!strcmp (module, "pmodeler"))  return "gmtpmodeler";
@@ -126,6 +141,7 @@ const char *gmtlib_get_active_name (struct GMTAPI_CTRL *API, const char *module)
 	else if (!strncmp (module, "psternary",    9U)) return "ternary";
 	else if (!strncmp (module, "pscontour",    9U)) return "contour";
 	else if (!strncmp (module, "psbasemap",    9U)) return "basemap";
+	else if (!strncmp (module, "psbarb",       6U)) return "barb";
 	else if (!strncmp (module, "psevents",     8U)) return "events";
 	else if (!strncmp (module, "pswiggle",     8U)) return "wiggle";
 	else if (!strncmp (module, "pslegend",     8U)) return "legend";
@@ -178,6 +194,7 @@ bool gmtlib_is_modern_name (struct GMTAPI_CTRL *API, const char *module) {
 	else if (!strncmp (module, "clip",       6U)) is_modern = true;
 	else if (!strncmp (module, "plot3d",     5U)) is_modern = true;
 	else if (!strncmp (module, "plot",       4U)) is_modern = true;
+	else if (!strncmp (module, "barb",       4U)) is_modern = true;
 	else if (!strncmp (module, "sac",        3U)) is_modern = true;
 	return is_modern;
 }
