@@ -1,6 +1,6 @@
 /*--------------------------------------------------------------------
  *
- *	Copyright (c) 1991-2022 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
+ *	Copyright (c) 1991-2024 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -38,24 +38,26 @@
  * These functions simplifies the access to the GMT shoreline, border, and river
  * databases.
  *
- * The PUBLIC functions are (16):
+ * A) List of exported gmt_* functions available to modules and libraries via gmt_dev.h:
  *
- * gmt_set_levels           : Modifies what items to extract from GSHHG database
- * gmt_set_resolution       : Converts resolutions f,h,i,l,c to integers 0-4
- * gmt_init_shore           : Opens selected shoreline database and initializes structures
- * gmt_get_shore_bin        : Returns all selected shore data for this bin
- * gmt_init_br              : Opens selected border/river database and initializes structures
- * gmt_get_br_bin           : Returns all selected border/river data for this bin
- * gmt_get_gshhg_lines      : Returns a GMT_DATASET with lines
- * gmt_assemble_shore       : Creates polygons or lines from shoreline segments
- * gmt_prep_shore_polygons  : Wraps polygons if necessary and prepares them for use
- * gmt_shore_level_at_point : Return hierarchical level at specified point
- * gmt_assemble_br          : Creates lines from border or river segments
- * gmt_free_shore           : Frees up memory used by shorelines for this bin
- * gmt_free_br              : Frees up memory used by shorelines for this bin
- * gmt_free_shore_polygons  : Frees list of polygon coordinates
- * gmt_shore_cleanup        : Frees up main shoreline structure memory
- * gmt_br_cleanup           : Frees up main river/border structure memory
+ *	gmt_assemble_br          : Creates lines from border or river segments
+ *	gmt_assemble_shore       : Creates polygons or lines from shoreline segments
+ *	gmt_br_cleanup           : Frees up main river/border structure memory
+ *	gmt_free_shore           : Frees up memory used by shorelines for this bin
+ *	gmt_free_br              : Frees up memory used by shorelines for this bin
+ *	gmt_free_shore_polygons  : Frees list of polygon coordinates
+ *	gmt_get_br_bin           : Returns all selected border/river data for this bin
+ *	gmt_get_shore_bin        : Returns all selected shore data for this bin
+ *	gmt_get_gshhg_lines      : Returns a GMT_DATASET with lines
+ *	gmt_init_shore           : Opens selected shoreline database and initializes structures
+ *	gmt_init_br              : Opens selected border/river database and initializes structures
+ *	gmt_prep_shore_polygons  : Wraps polygons if necessary and prepares them for use
+ *	gmt_set_levels           : Modifies what items to extract from GSHHG database
+ *	gmt_set_resolution       : Converts resolutions f,h,i,l,c to integers 0-4
+ *	gmt_shore_adjust_res     : Return highest available resolution
+ *	gmt_shore_cleanup        : Frees up main shoreline structure memory
+ *	gmt_shore_level_at_point : Return hierarchical level at specified point
+ *	gmt_shore_version        : Return the GSHHS version string
  *
  * Author:	Paul Wessel
  * Date:	1-JAN-2010
@@ -510,7 +512,7 @@ int gmt_set_resolution (struct GMT_CTRL *GMT, char *res, char opt) {
 	switch (*res) {
 		case 'a':	/* Automatic selection via -J or -R, if possible */
 			if (GMT->common.J.active && !gmt_M_is_linear (GMT)) {	/* Use map scale xxxx as in 1:xxxx */
-				double i_scale = 1.0 / (0.0254 * GMT->current.proj.scale[GMT_X]);
+				double i_scale = 1.0 / (0.0254 * fabs(GMT->current.proj.scale[GMT_X]));
 				if (i_scale > GMT_CRUDE_THRESHOLD)
 					base = 4;	/* crude */
 				else if (i_scale > GMT_LOW_THRESHOLD)
@@ -856,7 +858,11 @@ int gmt_get_shore_bin (struct GMT_CTRL *GMT, unsigned int b, struct GMT_SHORE *c
 			ID = c->GSHHS_node[node];	/* GSHHS Id of the polygon that determined the level of the current node */
 			while (ID >= 0 && c->node_level[k] && c->GSHHS_area[ID] < c->min_area) {	/* Polygon must be skipped and node level reset */
 				ID = c->GSHHS_parent[ID];	/* Pick the parent polygon since that is the next polygon up */
-				if (c->node_level[k] != c->node_level_g[k])
+				if (c->lat_sw < -60.0) {	/* Special check for Antarctica due to the two versions of the continent */
+					if (c->node_level[k] != c->node_level_g[k])
+						c->node_level[k]--;		/* ...and drop down one level to that of the parent polygon */
+				}
+				else	/* Not in Antarctica and we need to drop the level as we lost that polygon that covered this node */
 					c->node_level[k]--;		/* ...and drop down one level to that of the parent polygon */
 			}	/* Keep doing this until the polygon containing the node is "too big to fail" or we are in the ocean */
 		}
