@@ -1,6 +1,6 @@
 /*--------------------------------------------------------------------
  *
- *	Copyright (c) 1991-2021 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
+ *	Copyright (c) 1991-2024 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -36,6 +36,13 @@
  * GMT TYPE DEFINITIONS
  *--------------------------------------------------------------------*/
 
+/*! Definition of GMT_SCALED_RECT_DIM used to handle embellishments dimensions given by default or percentages of map width */
+struct GMT_SCALED_RECT_DIM {
+	bool fraction[2];	/* True if dimensions in dim are given as fraction of map width and rectangle height */
+	double dim[2];		/* Dimensions in inches, if set; */
+	double scl[2];		/* Scales set to give dim once map width is known (scl[GMT_Y] is relative to rectangle width, not map width) */
+};
+
 /*! Definition of CONTOUR_ARGS used by grdcontour and pscontour */
 struct CONTOUR_ARGS {
 	bool cpt;		/* true of we were given a CPT file */
@@ -66,11 +73,13 @@ struct GMT_MATH_MACRO {
 struct GMT_KEYWORD_DICTIONARY {	/* Used for keyword-value lookup */
 	char separator;			/* Single character separating 2 or more identical specifications [0 for no repeat] */
 	char short_option;		/* Single character GMT option code */
-	char long_option[GMT_LEN32-1];		/* Name of corresponding long option */
-	char short_directives[GMT_LEN32];	/* Single character directives, comma-separated */
+	char long_option[GMT_LEN256-1];		/* Name of corresponding long option */
+	char short_directives[GMT_LEN64];	/* Single character directives, comma-separated */
 	char long_directives[GMT_LEN256];	/* Long name directives, comma-separated */
-	char short_modifiers[GMT_LEN32];	/* Single character modifiers, comma-separated */
+	char short_modifiers[GMT_LEN64];	/* Single character modifiers, comma-separated */
 	char long_modifiers[GMT_LEN256];	/* Long name modifiers, comma-separated */
+	unsigned int transproc_mask;	/* Translation processing mask indicating special
+                                           behavior, e.g., multi-directive, etc., support */
 };
 
 /*! Definition of structure use for finding optimal n_columns/n_rows for surface */
@@ -129,6 +138,7 @@ struct GMT_ARRAY {	/* Used by modules that needs to set up 1-D output/bin arrays
 	bool reciprocal;	/* true if we gave the reciprocal increment */
 	bool round;	/* true if we want to adjust increment to ensure min/max range is a multiple of inc */
 	bool exact_inc;	/* true if we want the increment to be exact and to adjust min/max instead */
+	bool has_inc;	/* true if parsing detected an increment */
 	bool var_inc;	/* true if the resulting array has variable spacing */
 	bool logarithmic;	/* true if inc = 1,2,3 and we want logarithmic scale */
 	bool logarithmic2;	/* true if inc = integer and we want log2 scale */
@@ -260,6 +270,7 @@ struct GMT_MAP {		/* Holds all map-related parameters */
 	unsigned int n_lon_nodes;		/* Somewhat arbitrary # of nodes for lines in longitude (may be reset in gmt_map.c) */
 	unsigned int n_lat_nodes;		/* Somewhat arbitrary # of nodes for lines in latitude (may be reset in gmt_map.c) */
 	unsigned int path_mode;		/* 0 if we should call gmt_fix_up_path to resample across gaps > path_step, 1 to leave alone */
+	unsigned int last_dim;		/* 2 for 2-D or 3 for 3-D as previous plot */
 	double last_width;			/* Full width in inches of previous plot */
 	double last_height;			/* Full height in inches of previous plot */
 	double width;				/* Full width in inches of this world map */
@@ -383,11 +394,9 @@ struct GMT_CURRENT {
 	struct GMT_PSL ps;		/* Hold parameters related to PSL setup */
 	struct GMT_OPTION *options;	/* Pointer to current program's options */
 	struct GMT_FFT_HIDDEN fft;	/* Structure with info that must survive between FFT calls */
-#ifdef HAVE_GDAL
 	struct GMT_GDALREAD_IN_CTRL  gdal_read_in;  /* Hold parameters related to options transmitted to gdalread */
 	struct GMT_GDALREAD_OUT_CTRL gdal_read_out; /* Hold parameters related to options transmitted from gdalread */
 	struct GMT_GDALWRITE_CTRL    gdal_write;    /* Hold parameters related to options transmitted to gdalwrite */
-#endif
 };
 
 struct GMT_INTERNAL {
@@ -461,13 +470,13 @@ struct GMT_SESSION {
 
 struct GMT_CTRL {
 	/* Master structure for a GMT invocation.  All internal settings for GMT is accessed here */
+	struct PSL_CTRL *PSL;		/* Pointer to the PSL structure [or NULL] */
+	struct GMTAPI_CTRL *parent;	/* Owner of this structure [or NULL]; gives access to the API from functions being passed *GMT only */
 	struct GMT_SESSION session;	/* Structure with all values that do not change throughout a session */
 	struct GMT_INIT init;		/* Structure with all values that do not change in a GMT_func call */
 	struct GMT_COMMON common;	/* Structure with all the common GMT command settings (-R -J ..) */
 	struct GMT_CURRENT current;	/* Structure with all the GMT items that can change during execution, such as defaults settings (pens, colors, fonts.. ) */
 	struct GMT_INTERNAL hidden;	/* Internal global variables that are not to be changed directly by users */
-	struct PSL_CTRL *PSL;		/* Pointer to the PSL structure [or NULL] */
-	struct GMTAPI_CTRL *parent;	/* Owner of this structure [or NULL]; gives access to the API from functions being passed *GMT only */
 };
 
 /* p_to_io_func is used as a pointer to functions such as GMT_read_d in assignments
