@@ -69,6 +69,8 @@
 "n",
 "o",
 "p",
+"q",
 "s",
 "t",
+"w",
 "x"
