@@ -1,6 +1,6 @@
 /*--------------------------------------------------------------------
  *
- *	Copyright (c) 1991-2022 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
+ *	Copyright (c) 1991-2024 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -25,6 +25,7 @@
  */
 
 #include "gmt_dev.h"
+#include "longopt/gmtdefaults_inc.h"
 
 #define THIS_MODULE_CLASSIC_NAME	"gmtdefaults"
 #define THIS_MODULE_MODERN_NAME	"gmtdefaults"
@@ -65,7 +66,7 @@ static int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "  OPTIONAL ARGUMENTS:\n");
 	GMT_Usage (API, 1, "\n-D[s|u]");
 	GMT_Usage (API, -2, "Print the current GMT default settings. Optionally append a directive:");
-	GMT_Usage (API, 3, "s: Print the SI version of the system defaults.");
+	GMT_Usage (API, 3, "s: Print the SI version of the system defaults [Default].");
 	GMT_Usage (API, 3, "u: Print the US version of the system defaults.");
 	GMT_Usage (API, -2, "Note: ALL settings will be written to standard output.");
 	GMT_Option (API, "V");
@@ -95,7 +96,13 @@ static int parse (struct GMT_CTRL *GMT, struct GMTDEFAULTS_CTRL *Ctrl, struct GM
 
 			case 'D':	/* Get GMT system-wide defaults settings */
 				n_errors += gmt_M_repeated_module_option (API, Ctrl->D.active);
-				n_errors += gmt_get_required_char (GMT, opt->arg, opt->option, 0, &Ctrl->D.mode);
+				if (opt->arg[0]) {	/* Specified an argument */
+					n_errors += gmt_get_required_char (GMT, opt->arg, opt->option, 0, &Ctrl->D.mode);
+					if (strchr ("su", Ctrl->D.mode) == NULL) {
+						GMT_Report (API, GMT_MSG_ERROR, "Option -D: Argument %s is not recognized.\n", opt->arg);
+						n_errors++;
+					}
+				}
 				break;
 			case 'L':	/* List the user's current GMT defaults settings */
 				if (gmt_M_compat_check (GMT, 4)) {
@@ -120,8 +127,6 @@ static int parse (struct GMT_CTRL *GMT, struct GMTDEFAULTS_CTRL *Ctrl, struct GM
 #define bailout(code) {gmt_M_free_options (mode); return (code);}
 #define Return(code) {Free_Ctrl (GMT, Ctrl); gmt_end_module (GMT, GMT_cpy); bailout (code);}
 
-EXTERN_MSC void gmtinit_update_keys (struct GMT_CTRL *GMT, bool arg);
-
 EXTERN_MSC int GMT_gmtdefaults (void *V_API, int mode, void *args) {
 	int error;
 
@@ -140,7 +145,7 @@ EXTERN_MSC int GMT_gmtdefaults (void *V_API, int mode, void *args) {
 
 	/* Parse the command-line arguments */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, NULL, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, module_kw, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
@@ -157,7 +162,7 @@ EXTERN_MSC int GMT_gmtdefaults (void *V_API, int mode, void *args) {
 
 	/* To ensure that all is written to stdout we must set updated to true */
 
-	gmtinit_update_keys (GMT, true);
+	gmt_update_keys (GMT, true);
 
 	gmt_putdefaults (GMT, "-");
 
