@@ -1,6 +1,6 @@
 /*--------------------------------------------------------------------
  *
- *	Copyright (c) 1991-2022 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
+ *	Copyright (c) 1991-2024 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -24,6 +24,7 @@
  */
 
 #include "gmt_dev.h"
+#include "longopt/gmtinfo_inc.h"
 
 #define THIS_MODULE_CLASSIC_NAME	"gmtinfo"
 #define THIS_MODULE_MODERN_NAME	"gmtinfo"
@@ -87,7 +88,7 @@ struct GMTINFO_CTRL {	/* All control options for this program (except common arg
 	struct GMTINFO_L {	/* -L */
 		bool active;
 	} L;
-	struct GMTINFO_S {	/* -S[x|y] */
+	struct GMTINFO_S {	/* -S[x|y] [Deprecated in 6.5] */
 		bool active;
 		bool xbar, ybar;
 	} S;
@@ -127,7 +128,7 @@ static int usage (struct GMTAPI_CTRL *API, int level) {
 	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
 	GMT_Usage (API, 0, "usage: %s [<table>] [-Aa|t|s] [-C] [-D[<dx>[/<dy>]]] [-EL|l|H|h[<col>]] "
-		"[-Fi|d|t] [-I[b|e|f|p|s]<dx>[/<dy>[/<dz>..]][+e|r|R<incs>]] [-L] [-S[x][y]] [-T<dz>[%s][+c<col>]] "
+		"[-Fi|d|t] [-I[b|e|f|p|s]<dx>[/<dy>[/<dz>..]][+e|r|R<incs>]] [-L] [-T<dz>[%s][+c<col>]] "
 		"[%s] [%s] [%s] [%s] [%s] [%s] [%s] [%s] [%s] [%s] [%s] [%s] [%s] [%s] [%s] [%s]",
 		name, GMT_TIME_FIX_UNITS_DISPLAY, GMT_V_OPT, GMT_a_OPT, GMT_bi_OPT, GMT_d_OPT, GMT_e_OPT, GMT_f_OPT, GMT_g_OPT, GMT_h_OPT,
 		GMT_i_OPT, GMT_o_OPT, GMT_qi_OPT, GMT_r_OPT, GMT_s_OPT, GMT_w_OPT, GMT_colon_OPT, GMT_PAR_OPT);
@@ -152,27 +153,25 @@ static int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Usage (API, 3, "d: Dataset: One record per segment with tbl_no, seg_no, nrows, start_rec, stop_rec.");
 	GMT_Usage (API, 3, "t: Tables:  Same as D but the counts resets per table.");
 	GMT_Usage (API, 1, "\n-I[b|e|f|p|s]<dx>[/<dy>[/<dz>..]][+e|r|R<incs>]");
-	GMT_Usage (API, -2, "Return textstring -Rw/e/s/n to nearest multiple of <dx>/<dy> (assumes at least two columns). "
-		"Give -Ie to just report the min/max extent in the -Rw/e/s/n string (no multiples). Give -I<dx>/0 or -I0/<dy> for mixed exact and rounded result. "
-		"If -C is set then no -R string is issued.  Instead, the number of increments "
-		"given determines how many columns are rounded off to the nearest multiple. "
-		"If only one increment is given we also use it for the second column (for backwards compatibility). "
-		"To override this behavior, use -Ip<dx>. "
-		"If input data are regularly distributed we use observed phase shifts in determining -R [no phase shift] "
-		"and allow -r to change from gridline-registration to pixel-registration. "
-		"Use -Ib to report the bounding box polygon for the data files (or segments; see -A). "
-		"Use -If<dx>[/<dy>] to report an extended region optimized for fastest results in FFTs. "
-		"Use -Is<dx>[/<dy>] to report an extended region optimized for fastest results in surface. "
-		"Append +r to modify the region further: Append <inc>, <xinc>/<yinc>, or <winc>/<einc>/<sinc>/<ninc> "
-		"to round region to these multiples; use +R to extend region by those increments instead, "
-		"or use +e which is like +r but makes sure the region extends at least by %g x <inc>.\n", GMT_REGION_INCFACTOR);
+	GMT_Usage (API, -2, "Return textstring -Rw/e/s/n to nearest multiple of <dx>/<dy> (assumes input has at least two columns). "
+		"Several directives affect the result: ");
+	GMT_Usage (API, 3, "%s b: Report the bounding box polygon for the data files (or segments; see -A).", GMT_LINE_BULLET);
+	GMT_Usage (API, 3, "%s e: Report the exact min/max extent in the -Rw/e/s/n string (no multiples). Give -I<dx>/0 or -I0/<dy> for mixed exact and rounded result.", GMT_LINE_BULLET);
+	GMT_Usage (API, 3, "%s f: Append <dx>[/<dy>] to report an extended region optimized for fastest results in FFTs.", GMT_LINE_BULLET);
+	GMT_Usage (API, 3, "%s p: If only one increment is given we also use it for the second column (for backwards compatibility). "
+		"To override this behavior, use -Ip<dx>.", GMT_LINE_BULLET);
+	GMT_Usage (API, 3, "%s s: Append <dx>[/<dy>] to report an extended region optimized for fastest results in surface.", GMT_LINE_BULLET);
+	GMT_Usage (API, -2, "Three modifiers can adjust the determined region further:");
+	GMT_Usage (API, 3, "+r: Modify the region further: Append <inc>, <xinc>/<yinc>, or <winc>/<einc>/<sinc>/<ninc> "
+		"to round region to these multiples.");
+	GMT_Usage (API, 3, "+e: Like +r, but makes sure the region extends at least by %g x <inc>.", GMT_REGION_INCFACTOR);
+	GMT_Usage (API, 3, "+R: Like +r but adds and subtracts the given increments to extend the region outwards.");
+	GMT_Usage (API, -2, "Notes: (1) If -C is set then no -R string is issued.  Instead, the number of increments "
+		"given determines how many columns are rounded off to the nearest multiple.");
+	GMT_Usage (API, -2, "Notes: (2) If input data are regularly distributed we use observed phase shifts in determining "
+		"-R [no phase shift] and allow -r to change from gridline-registration to pixel-registration.");
 	GMT_Usage (API, 1, "\n-L Determine limiting region. With -I it rounds inward so bounds are within data range. "
 		"Use -A to find the limiting common bounds of all segments or tables.");
-	GMT_Usage (API, 1, "\n-S[x][y]");
-	GMT_Usage (API, -2, "Add extra space for error bars. Useful together with -I.");
-	GMT_Usage (API, 3, "-Sx: Leaves space for horizontal error bar using value in third (2) column.");
-	GMT_Usage (API, 3, "-Sy: Leaves space for vertical error bar using value in third (2) column.");
-	GMT_Usage (API, 3, "-S or -Sxy: Leaves space for both error bars using values in third&fourth (2&3) columns.");
 	GMT_Usage (API, 1, "\n-T<dz>[%s][+c<col>]", GMT_TIME_FIX_UNITS_DISPLAY);
 	GMT_Usage (API, -2, "Return textstring -Tzmin/zmax/dz to nearest multiple of the given <dz>.");
 	GMT_Usage (API, -2, "Note: Calculations are based on the first (0) column; append +c<col> to use another column. "
@@ -324,7 +323,7 @@ static int parse (struct GMT_CTRL *GMT, struct GMTINFO_CTRL *Ctrl, struct GMT_OP
 				n_errors += gmt_M_repeated_module_option (API, Ctrl->L.active);
 				n_errors += gmt_get_no_argument (GMT, opt->arg, opt->option, 0);
 				break;
-			case 'S':	/* Error bar output */
+			case 'S':	/* Error bar output [deprecated in 6.5] */
 				n_errors += gmt_M_repeated_module_option (API, Ctrl->S.active);
 				j = 0;
 				while (opt->arg[j]) {
@@ -410,7 +409,7 @@ static int parse (struct GMT_CTRL *GMT, struct GMTINFO_CTRL *Ctrl, struct GMT_OP
 #define Return(code) {Free_Ctrl (GMT, Ctrl); gmt_M_free (GMT, xyzmin); gmt_M_free (GMT, xyzmax); gmt_M_free (GMT, xyzminL); gmt_M_free (GMT, lonmin);  gmt_M_free (GMT, lonmax); gmt_M_free (GMT, xyzmaxL); gmt_M_free (GMT, Q); gmt_M_free (GMT, Z); gmt_M_free (GMT, Out); gmt_end_module (GMT, GMT_cpy); bailout (code);}
 
 EXTERN_MSC int GMT_gmtinfo (void *V_API, int mode, void *args) {
-	bool got_stuff = false, first_data_record, give_r_string = false, save_t;
+	bool got_stuff = false, first_data_record, give_r_string = false, save_t, first_time = true;
 	bool brackets = false, work_on_abs_value, do_report, done, full_range = false;
 	int i, j, error = 0, col_type[GMT_MAX_COLUMNS];
 	unsigned int fixed_phase[2] = {1, 1}, min_cols, save_range, n_items = 0;
@@ -442,7 +441,7 @@ EXTERN_MSC int GMT_gmtinfo (void *V_API, int mode, void *args) {
 
 	/* Parse the command-line arguments */
 
-	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, NULL, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, module_kw, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
 	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
@@ -529,6 +528,7 @@ EXTERN_MSC int GMT_gmtinfo (void *V_API, int mode, void *args) {
 	delimiter[0] = (Ctrl->C.active || Ctrl->I.mode == BOUNDBOX) ? '\t' : '/';
 	delimiter[1] = '\0';
 	off = (GMT->common.R.active[GSET]) ? 0.5 : 0.0;
+	save_range = GMT->current.io.geo.range;
 
 	brackets = !Ctrl->C.active;
 	work_on_abs_value = (Ctrl->E.active && Ctrl->E.abs);
@@ -565,7 +565,6 @@ EXTERN_MSC int GMT_gmtinfo (void *V_API, int mode, void *args) {
 		Return (API->error);
 	}
 
-	save_range = GMT->current.io.geo.range;
 	GMT->current.io.geo.range = GMT_IGNORE_RANGE;	/* Ensure no adjustment will happen since we control it here */
 	d_ptr = (Ctrl->C.active || Ctrl->E.active) ? GMT->current.io.curr_rec : ((Ctrl->I.mode == BOUNDBOX) ? out : NULL);
 	t_ptr = (d_ptr && !Ctrl->E.active) ? NULL : record;
@@ -590,7 +589,10 @@ EXTERN_MSC int GMT_gmtinfo (void *V_API, int mode, void *args) {
 			gmt_quad_reset (GMT, Q, ncol);
 			if (gmt_M_rec_is_segment_header (GMT) && Ctrl->A.mode != REPORT_PER_SEGMENT) continue;	/* Since we are not reporting per segment they are just headers as far as we are concerned */
 		}
-
+		if (first_time && gmt_M_is_geographic (GMT, GMT_IN)) {	/* Learned that an OGR file is geographic so output is the same */
+			gmt_set_geographic (GMT, GMT_OUT);
+			first_time = false;
+		}
 		if (gmt_M_rec_is_segment_header (GMT) || (Ctrl->A.mode == REPORT_PER_TABLE && gmt_M_rec_is_file_break (GMT)) || gmt_M_rec_is_eof (GMT)) {	/* Time to report */
 			if (Ctrl->A.active) {
 				if (Ctrl->A.mode == REPORT_PER_TABLE && gmt_M_rec_is_file_break (GMT)) {
@@ -611,9 +613,11 @@ EXTERN_MSC int GMT_gmtinfo (void *V_API, int mode, void *args) {
 
 			do_report = true;
 			if (Ctrl->L.active && !gmt_M_rec_is_eof (GMT)) do_report = false;	/* Only final report for -L */
- 			for (col = 0; col < ncol; col++) if (GMT->current.io.col_type[GMT_IN][col] == GMT_IS_LON) {	/* Must update longitudes separately */
-				gmt_find_range (GMT, Z[col], n_items, &xyzmin[col], &xyzmax[col]);
-				n_items = 0;
+ 			for (col = 0; col < ncol; col++) {		/* Must update longitudes separately */
+				if (GMT->current.io.col_type[GMT_IN][col] == GMT_IS_LON) {
+					gmt_find_range (GMT, Z[col], n_items, &xyzmin[col], &xyzmax[col]);
+					n_items = 0;
+				}
 			}
 			if (Ctrl->I.active) {	/* Must report multiples of dx/dy etc */
 				if (n > 1 && fixed_phase[GMT_X] && fixed_phase[GMT_Y]) {	/* Got xy[z] data that lined up on a grid, so use the common phase shift */
@@ -710,6 +714,20 @@ EXTERN_MSC int GMT_gmtinfo (void *V_API, int mode, void *args) {
 						ww, ee, ss, nn, in_dim[GMT_X], in_dim[GMT_Y], wesn[XLO], wesn[XHI], wesn[YLO], wesn[YHI], out_dim[GMT_X], out_dim[GMT_Y]);
 				}
 			}
+
+			if (gmt_M_is_geographic (GMT, GMT_OUT)) {
+				/* Special handling since lon1/lon2 must first have been ensured (above) to satisfy lon2 > lon1, but
+			 	* we may wish for a specific range format, so listen to what save_range was before we set it to GMT_IGNORE_RANGE */
+				if (wesn[XLO] > 0.0 && save_range == GMT_IS_M180_TO_P180_RANGE) {
+					wesn[XLO] -= 360.0;	wesn[XHI] -= 360.0;
+				}
+				else if (save_range == GMT_IS_0_TO_P360_RANGE && wesn[XHI] < 0.0) {
+					wesn[XLO] += 360.0;	wesn[XHI] += 360.0;
+				}
+				else if (save_range == GMT_IS_M360_TO_0_RANGE && wesn[XHI] > 0.0) {
+					wesn[XLO] -= 360.0;	wesn[XHI] -= 360.0;
+				}
+			}
 			if (give_r_string) {	/* Return -R string */
 				if (full_range)
 					sprintf (record, "-R%g/%g/", wesn[XLO], wesn[XHI]);
